(*  Copyright (C) 1999 Greatis Software  *)
(*    web: http://www.greatissoft.com    *)
(*    e-mail: support@greatissoft.com    *)

unit NameProp;

interface

uses Classes, Forms, Controls, DsgnIntf, SysUtils, Dialogs, Registry;

type
  TNameProperty=class(TComponentNameProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

procedure Register;

implementation

uses NameEdit;

const
  RegName = 'SOFTWARE\Greatis';
  RegSecName = 'NameProp';
  OptionsSecName = RegSecName+'\Options';
  PrefixesSecName = RegSecName+'\Prefixes';
  HistorySecName = RegSecName+'\History';
  MaxHistory=100;

var
  List,History: TStringList;

procedure TNameProperty.Edit;
var
  S: TCaption;
  Idx: Integer;
begin
  with TfrmNameEditor.Create(Application) do
  try
    lsbPrefixes.Items.Assign(List);
    cmbName.Items.Assign(History);
    cmbName.MaxLength:=GetEditLimit;

    if TComponent(GetComponent(0)).Owner<>nil then
      S:=TComponent(GetComponent(0)).Owner.Name+'.'
    else S:='';
    Caption:=S+TComponent(GetComponent(0)).Name+'.'+GetName;

    S:=GetComponent(0).ClassName;
    NativeType:=S;

    with List do
    begin
      Idx:=IndexOf(Copy(S,2,Pred(Length(S))));
      if Idx<>-1 then
      begin
        lblPrefix.Caption:=TPrefix(Objects[Idx]);
        lblClassName.Caption:=S;
        ComponentType:=S;
      end
      else
      begin
        S:=GetComponent(0).ClassParent.ClassName;
        Idx:=IndexOf(Copy(S,2,Pred(Length(S))));
        if Idx<>-1 then
        begin
          lblPrefix.Caption:=TPrefix(Objects[Idx]);
          lblClassName.Caption:='Inherited from '+S;
          ComponentType:=S;
        end
        else
        begin
          lblPrefix.Caption:='???';
          lblClassName.Caption:=NativeType;
          btnOK.Enabled:=False;
        end;
      end;
    end;

    S:=TComponent(GetComponent(0)).Name;
    if Copy(S,1,3)=lblPrefix.Caption then
      cmbName.Text:=Copy(S,4,Length(S)-3);

    ShowModal;
    if ModalResult=mrOK then
    begin
      SetValue(lblPrefix.Caption+cmbName.Text);
      with cmbName.Items,cmbName do
      begin
        if (Count>0) and (IndexOf(Text)<>-1) then Move(IndexOf(Text),0)
        else Insert(0,Text);
        if Count>MaxHistory then Delete(Pred(Count));
      end;
    end;
    List.Assign(lsbPrefixes.Items);
    History.Assign(cmbName.Items);
  finally
    Free;
  end;
end;

function TNameProperty.GetAttributes: TPropertyAttributes;
begin
  Result:=inherited GetAttributes+[paDialog];
end;

procedure CreateList;
var
  i: Integer;
  L: TStringList;
begin
  List:=TStringList.Create;
  L:=TStringList.Create;
  with TRegIniFile.Create(RegName) do
  try
    ReadSectionValues(PrefixesSecName,L);
    for i:=0 to Pred(L.Count) do
      List.AddObject(L.Names[i],Pointer(TPrefix(L.Values[L.Names[i]])));
  finally
    L.Free;
    Free;
  end;
end;

procedure FreeList;
var
  i: Integer;
begin
  with TRegIniFile.Create(RegName) do
  try
    with List do
    try
      for i:=0 to Pred(Count) do
        WriteString(PrefixesSecName,Strings[i],TPrefix(Objects[i]));
    finally
      Free;
    end;
  finally
    Free;
  end;
end;

procedure ReadConfig;
var
  i,Cnt: Integer;
begin
  with TRegIniFile.Create(RegName) do
  try
    WriteString(RegSecName,'@Name','Component Name Property Editor');
    WriteString(RegSecName,'@Path','');
    WriteString(RegSecName,'@Version','3');
    DlgLeft:=ReadInteger(OptionsSecName,'DlgLeft',200);
    DlgTop:=ReadInteger(OptionsSecName,'DlgTop',140);
    Cnt:=ReadInteger(HistorySecName,'Count',0);
    History:=TStringList.Create;
    for i:=1 to Cnt do
      History.Add(ReadString(HistorySecName,IntToStr(i),''));
  finally
    Free;
  end;
end;

procedure WriteConfig;
var
  i: Integer;
begin
  with TRegIniFile.Create(RegName) do
  try
    WriteInteger(OptionsSecName,'DlgLeft',DlgLeft);
    WriteInteger(OptionsSecName,'DlgTop',DlgTop);
    WriteInteger(HistorySecName,'Count',History.Count);
    for i:=1 to History.Count do
      WriteString(HistorySecName,IntToStr(i),History[Pred(i)]);
  finally
    Free;
  end;
end;

procedure Register;
begin
  RegisterPropertyEditor(TypeInfo(TComponentName),TComponent,'Name',TNameProperty);
end;

initialization
  CreateList;
  ReadConfig;
finalization
  WriteConfig;
  FreeList;
end.
